getGapSizePlot <- function() {
  
  refactorableUsingLambda <- getLambdaRefactorableTable()
  
  expressionGapInfo <- allExpressionGapsInfo
  expressionGapInfo$X.Statements1 <- 1
  expressionGapInfo$X.Statements2 <- 1
  expressionGapInfo <- merge(expressionGapInfo, refactorableUsingLambda, by=KEYS)[, c(KEYS, "X.StatementsInCloneFragment1", "X.StatementsInCloneFragment2","X.Statements1", "X.Statements2")]
  expressionGapInfo$Left <- expressionGapInfo$X.Statements1 / expressionGapInfo$X.StatementsInCloneFragment1
  expressionGapInfo$Right <- expressionGapInfo$X.Statements2 / expressionGapInfo$X.StatementsInCloneFragment2
  expressionGapInfoCombined <- data.frame(Data = c(expressionGapInfo$Left, expressionGapInfo$Right), Group="Expression Diffs")
  
  blockGapInfo <- allBlockGapsInfo
  blockGapInfo <- merge(blockGapInfo, refactorableUsingLambda, by=KEYS)[, c(KEYS, "X.StatementsInCloneFragment1", "X.StatementsInCloneFragment2","X.Statements1", "X.Statements2")]
  blockGapInfo$Left <- blockGapInfo$X.Statements1 / blockGapInfo$X.StatementsInCloneFragment1
  blockGapInfo$Right <- blockGapInfo$X.Statements2 / blockGapInfo$X.StatementsInCloneFragment2
  blockGapInfoCombined <- data.frame(Data = c(blockGapInfo$Left, blockGapInfo$Right), Group="Statement Gaps")
  blockGapInfoCombined <- blockGapInfoCombined[blockGapInfoCombined$Data <= 1,]
  
  print("Expression diffs size / clone fragment size:")
  print(summary(expressionGapInfoCombined$Data))
  print("Block gap size / clone fragment size:")
  print(summary(blockGapInfoCombined$Data))
  
  gapsInfo <- rbind(expressionGapInfoCombined, blockGapInfoCombined)

  splitted <- split(gapsInfo$Data, gapsInfo$Group)
  medianValues <- c()
  for (s in splitted) {
    medianValues <- c(medianValues, median(s))
  }

  ggplot(data = gapsInfo, aes(x = Group, y = Data)) +
    scale_color_grey() + 
    geom_violin(trim = F, fill = '#efefef', color = "#aaaaaa", weight = 0.5) + 
    geom_boxplot(width = 0.6, fill = "#444444", color = "#555555", alpha = 5/10, outlier.shape = NA) + 
    theme_bw() +
    theme(axis.ticks = element_blank(), axis.title.x = element_blank(), axis.title.y = element_blank()) + 
    geom_segment(aes(x = 0.5, xend = 1.5, y = medianValues[1], yend = medianValues[1]), colour="#555555", size = 0.2) + 
    geom_segment(aes(x = 1.5, xend = 2.5, y = medianValues[2], yend = medianValues[2]), colour="#555555", size = 0.2) + 
    scale_y_continuous("", breaks = c(min(gapsInfo$Data), max(gapsInfo$Data), medianValues), labels = function(x) { format(x, digits = 2, scientific = FALSE) } )
}

getGapsInfo <- function() {
  lambdaRefactorable <- getLambdaRefactorableTable()
  
  #Select the same columns from the tables of block and expression gaps
  colNamesToSelect <- c(KEYS, "X.Params", "ReturnType",	"X.ThrownExceptions",	"X.NonEffectiveFinalVars")
  blockGapsInfo <- allBlockGapsInfo[,colNamesToSelect]
  blockGapsInfo <- merge(blockGapsInfo, lambdaRefactorable, BY = KEYS)
  blockGapsInfo$Type <- "Block"
  expressionGapsInfo <- allExpressionGapsInfo[,colNamesToSelect]
  expressionGapsInfo <- merge(expressionGapsInfo, lambdaRefactorable, BY = KEYS)
  expressionGapsInfo$Type <- "Expression"
  
  #Append two resulting tables and return
  return(rbind(blockGapsInfo, expressionGapsInfo))
}

getLambdaExpressionsInfo <- function() {
  
  gapsInfo <- getGapsInfo()
  
  cat(sprintf("All lambdas: %6d \n", nrow(gapsInfo)))
  cat(sprintf("Block gaps ratio: %6.3f \n", nrow(gapsInfo[gapsInfo$Type == "Block",]) / nrow(gapsInfo)))
  cat(sprintf("Expression gaps ratio: %6.3f \n", nrow(gapsInfo[gapsInfo$Type == "Expression",]) / nrow(gapsInfo)))
  cat(sprintf("Clone pairs having only expression gap: %s\n", nrow(unique(gapsInfo[gapsInfo$Type == "Expression", KEYS]))))
  cat(sprintf("Clone pairs having only block gap: %s\n", nrow(unique(gapsInfo[gapsInfo$Type == "Block", KEYS]))))
  
  ratioOfLambdasToHaveNonEffectivelyFinalVars <- nrow(gapsInfo[gapsInfo$X.NonEffectiveFinalVars > 0,]) / nrow(gapsInfo)
  
  cat(sprintf("\nRatio of lambdas having non-effectively final vars: %6.3f \n", ratioOfLambdasToHaveNonEffectivelyFinalVars))
  cat("\n\nSummary of the number of non-effectively final vars:\n")
  print(summary(gapsInfo$X.NonEffectiveFinalVars))
  
  gapsInfo$Count <- 1
  aggregated <- aggregate(gapsInfo$Count, by=list(gapsInfo$GroupID, gapsInfo$PairID, gapsInfo$TreeID, gapsInfo$ProjectName), FUN="sum")
  
  cat("\nSummary of the number of lambda expressions per lambdafied clone pair\n")
  print(summary(aggregated$x)) #Number of lambda expressions
  
  med <- median(aggregated$x)
  breaks <- c(min(aggregated$x), max(aggregated$x), med, quantile(aggregated$x, .75))
  
  ggplot(data = aggregated, aes(x = 1, y = x)) +
      scale_color_grey() + 
      geom_violin(trim = F, fill = '#efefef', color = "#aaaaaa", weight = 0.5) + 
      geom_boxplot(width = 0.6, fill = "#444444", color = "#555555", alpha = 5/10, outlier.shape = NA) + 
      theme_bw() +
      theme(axis.ticks = element_blank(), axis.title.x = element_blank(), axis.title.y = element_blank()) + 
      geom_segment(aes(x = 0.5, xend = 1.5, y = med, yend = med), colour="#555555", size = 0.2) +
      scale_x_discrete(expand = c(0.25, 0)) +
      scale_y_log10(breaks = breaks,  labels = function(x) { format(x, digits = 2, scientific = FALSE) } )
}

getInterfaceTypeStatistics <- function(couldBe = F) {
  
  gapsInfo <- getGapsInfo()
  
  #Supplier: 0 params, returns something, no thrown exceptions
  supplierCondition <- gapsInfo$X.Params == 0 & gapsInfo$ReturnType != "void" & gapsInfo$X.ThrownExceptions == 0
  
  #Consumer: 1 param, doesn't return anything, no thrown exceptions
  consumerCondition <- gapsInfo$X.Params == 1 & gapsInfo$ReturnType == "void" & gapsInfo$X.ThrownExceptions == 0
  
  #Function interface: exactly one param, returns something, no thrown exceptions
  functionCondition <- gapsInfo$X.Params == 1 & gapsInfo$ReturnType != "void" & gapsInfo$X.ThrownExceptions == 0
  
  #BiConsumer
  biConsumerCondition <- gapsInfo$X.Params == 2 & gapsInfo$ReturnType == "void" & gapsInfo$X.ThrownExceptions == 0
  
  #BiFunction
  biFunctionCondition <- gapsInfo$X.Params == 2 & gapsInfo$ReturnType != "void" & gapsInfo$X.ThrownExceptions == 0
  
  #Custom functional interface: more than one param or having thrown exceptions
  customCondition <- (gapsInfo$X.Params > 2) | 
    (gapsInfo$X.Params == 0 & gapsInfo$ReturnType == "void" & gapsInfo$X.ThrownExceptions == 0) |
    (gapsInfo$X.ThrownExceptions > 0)
  
  gapsInfo$InterfaceType[supplierCondition] <- "Supplier"
  gapsInfo$InterfaceType[consumerCondition] <- "Consumer"
  gapsInfo$InterfaceType[functionCondition] <- "Function"
  gapsInfo$InterfaceType[customCondition] <- "Custom"
  #gapsInfo$InterfaceType[biConsumerCondition] <- "BiConsumer"
  #gapsInfo$InterfaceType[biFunctionCondition] <- "BiFunction"
  
  if (!couldBe) {
    
    mainTable <- table(gapsInfo$InterfaceType)
    mainTable <- mainTable / sum(mainTable) * 100
    print(mainTable)
    getInterfaceBarPlot((data.frame(InterfaceType = names(mainTable), Frequency = as.vector(mainTable))), ylimit = 50)
    
  } else {
    
    customsInfo <- gapsInfo[gapsInfo$InterfaceType == "Custom", ]
    
    couldBeSupplierCondition <- customsInfo$X.Params == 0 & customsInfo$ReturnType != "void" & customsInfo$X.ThrownExceptions > 0
    couldBeConsumerCondition <- customsInfo$X.Params == 1 & customsInfo$ReturnType == "void" & customsInfo$X.ThrownExceptions > 0
    couldBeFunctionCondition <- customsInfo$X.Params == 1 & customsInfo$ReturnType != "void" & customsInfo$X.ThrownExceptions > 0
    couldBeBiConsumerCondition <- customsInfo$X.Params == 2 & customsInfo$ReturnType == "void" & customsInfo$X.ThrownExceptions > 0
    couldBeBiFunctionCondition <- customsInfo$X.Params == 2 & customsInfo$ReturnType != "void" & customsInfo$X.ThrownExceptions > 0
    
    customsInfo$InterfaceType <- "Custom"
    customsInfo$InterfaceType[couldBeSupplierCondition] <- "Supplier"
    customsInfo$InterfaceType[couldBeConsumerCondition] <- "Consumer"
    customsInfo$InterfaceType[couldBeFunctionCondition] <- "Function"
    #customsInfo$InterfaceType[couldBeBiConsumerCondition] <- "Could be BiConsumer"
    #customsInfo$InterfaceType[couldBeBiFunctionCondition] <- "Could be BiFunction"
    
    customTable <- table(customsInfo$InterfaceType)
    customTable <- customTable / sum(customTable) * 100
    print(customTable)
    getInterfaceBarPlot((data.frame(InterfaceType = names(customTable), Frequency = as.vector(customTabl))))
    
  }
}

getInterfaceBarPlot <- function(table, yTitle = "", ylimit=75) {
  ggplot(table, aes(x = InterfaceType, y = Frequency)) +
    geom_bar(fill="#efefef", color="#aaaaaa", stat='identity') + 
    scale_color_grey() +
    scale_y_continuous(limits = c(0, ylimit)) +
    theme_bw() +
    theme(axis.ticks = element_blank(), axis.title.x = element_blank(), axis.title.y = if (yTitle == "") element_blank() else element_text(size = 14)) +
    scale_x_discrete(expand = c(0.05, 0)) +
    geom_text(aes(label=sprintf("%1.2f%%", Frequency)), vjust=-.5, position=position_dodge(.9), size=3)
}

getFrequencyOfReturnTypesBlock <- function() {
  getFrequencyOfReturnTypes(allBlockGapsInfo)
}

getFrequencyOfReturnTypesExpressionGaps <- function() {
  getFrequencyOfReturnTypes(allExpressionGapsInfo)
}

getFrequencyOfReturnTypes <- function(table) {
  temp <- table
  temp$Count <- 1
  returnTypes <- aggregate(temp$Count, by = list(temp$ReturnType), FUN="sum")
  print(returnTypes[order(-returnTypes$x),])
  #pie <- ggplot(temp, aes(x=1,y = factor(temp$ReturnType))) +
  #  geom_bar(width = 1, colour = "black")
  #pie + coord_polar()
}

getCloneGroupSizeStats <- function() {
  #table <- getLambdaRefactorableTable()
  table <- reportsTreesJoined
  table <- table[, c("GroupID", "PairID", "ProjectName")]
  aggregated <- aggregate(table$PairID, by=list(GroupID = table$GroupID, ProjectName = table$ProjectName), FUN=function(x) {
    splitted <- strsplit(as.character(x), "-", fixed = T)
    return(as.numeric(max(unlist(splitted))))
  })
  print("Summary of size of clone groups:")
  print(summary(aggregated$x))
  print("Number of outliers:")
  print(length(boxplot(aggregated$x)$out))
}

